#include <Arduino.h>

#include "SPI.h"
#include "TFT_eSPI.h"
#include "touchScreenPoint.h"
#include "../lvgl/src/lvgl.h"

#define HOR_RES 320
#define VER_RES 240

TFT_eSPI tft = TFT_eSPI();

TouchScrPoint oldPoint;

static lv_disp_buf_t disp_buf; //LVGL stuff;
static lv_color_t buf[LV_HOR_RES_MAX * 10];
lv_disp_drv_t disp_drv;
lv_indev_drv_t indev_drv;

static lv_obj_t *screenMain;
static lv_obj_t * kb; /* Teclat */
static lv_obj_t * ta; /* Àrea de text */

TouchScrPoint tftTouchGetPoint(){
  uint16_t x, y;
  TouchScrPoint tsp(-1,-1);

  if (tft.getTouch(&x, &y)) {
    tsp = TouchScrPoint(x,y);
  }

  return tsp;
}

void my_disp_flush(lv_disp_drv_t *disp, const lv_area_t *area, lv_color_t *color_p){
  uint32_t w = (area->x2 - area->x1 + 1);
  uint32_t h = (area->y2 - area->y1 + 1);

  tft.startWrite();
  tft.setAddrWindow(area->x1, area->y1, w, h);
  tft.pushColors(&color_p->full, w * h, true);
  tft.endWrite();

  lv_disp_flush_ready(disp);
}

bool my_input_read(lv_indev_drv_t * drv, lv_indev_data_t*data){
  uint16_t x, y;

  if (tft.getTouch(&x, &y)){
    data->state = LV_INDEV_STATE_PR;
    data->point.x = oldPoint.x = x;
    data->point.y = oldPoint.y = y;
    // String sz = "x: " + String(x,DEC) + " , y: " + String(y,DEC);
    // Serial.println(sz);   
  }else{
    data->state = LV_INDEV_STATE_REL;
    data->point.x = oldPoint.x;
    data->point.y = oldPoint.y;    
  }

  return false; 
}

static void kb_event_cb(lv_obj_t * keyboard, lv_event_t e){
  lv_keyboard_def_event_cb(kb, e);
  if(e == LV_EVENT_CANCEL) {
    lv_keyboard_set_textarea(kb, NULL);
    lv_obj_del(kb);
    kb = NULL;
  }
  if(e == LV_EVENT_APPLY){
    String szTextEscrit = lv_textarea_get_text(ta);
    Serial.println(szTextEscrit);
  }
}

static void kb_create(void){
  kb = lv_keyboard_create(lv_scr_act(), NULL);
  lv_keyboard_set_cursor_manage(kb, true);
  lv_obj_set_event_cb(kb, kb_event_cb);
  lv_keyboard_set_textarea(kb, ta);
}

static void ta_event_cb(lv_obj_t * ta_local, lv_event_t e){
  if(e == LV_EVENT_CLICKED && kb == NULL) {
    kb_create();
  }
}

void lv_ex_keyboard_1(void){
  /*Create a text area. The keyboard will write here*/
  ta  = lv_textarea_create(lv_scr_act(), NULL);
  //ta  = lv_textarea_create(scr, NULL);
  lv_obj_align(ta, NULL, LV_ALIGN_IN_TOP_MID, 0, LV_DPI / 16);
  lv_obj_set_event_cb(ta, ta_event_cb);
  lv_textarea_set_text(ta, "");
  lv_coord_t max_h = LV_VER_RES / 2 - LV_DPI / 8;
  if(lv_obj_get_height(ta) > max_h) lv_obj_set_height(ta, max_h);

  kb_create();
}

void setup() {
  Serial.begin(115200);

  tft.begin();
  tft.setRotation(1);
  oldPoint = tftTouchGetPoint();

  lv_init();
  lv_disp_buf_init(&disp_buf, buf, NULL, LV_HOR_RES_MAX * 10);
  /*Initialize the display*/
  lv_disp_drv_init(&disp_drv);
  disp_drv.hor_res = HOR_RES;
  disp_drv.ver_res = VER_RES;
  disp_drv.flush_cb = my_disp_flush;
  disp_drv.buffer = &disp_buf;
  lv_disp_drv_register(&disp_drv);

  /*Initialize the input device driver*/
  lv_indev_drv_init(&indev_drv);             /*Descriptor of a input device driver*/
  indev_drv.type = LV_INDEV_TYPE_POINTER;    /*Touch pad is a pointer-like device*/
  indev_drv.read_cb = my_input_read;         /*Set your driver function*/
  lv_indev_drv_register(&indev_drv);         /*Finally register the driver*/

  screenMain = lv_obj_create(NULL, NULL);
  lv_scr_load(screenMain); // Per a poder fer servir lv_scr_act() (Pantalla activa)

  lv_ex_keyboard_1();
}

void loop() {
  lv_task_handler();

  delay(1);
}